<?php

namespace App\Controller;

/**
 * Class Project
 * @package App\Controller
 * @author Artsiom Malkovich (skype: artmalk)
 */

class Project extends \App\Controller\Admin
{

    public function editAction()
    {
        $errors = array();
        /** @var \App\Model\Project $project */

        $project = \R::load('projects', 1);


        if ($project->id && $project->getLastParser() && $project->getLastParser()->getStatus() != \App\Model\Parser::STATUS_FINISH) {
            $errors['status'] = \t('PROJECT_CHANGE_PARSER_NOT_FINISHED');
        }

        if ($this->request()->isPost()) {
            $project->name = $this->request()->post('name');
            $project->link = $this->request()->post('link');
            if (!preg_match("/^[1-9a-zA-Z'-]{2,256}$/",$project->name)) {
                $errors['name'] =  'Invalid project name.';
            }
            if (!preg_match("/^(https?:\/\/)?([\da-z\.-]+)\.([a-z\.]{2,6})$/",$project->link)) {
                $errors['link'] =  'Invalid project link.';
            }
            if (!$errors) {
                if ($project->id) {
                    $this->app->flash('project-action', 'Project '.$project->name.' has been succesfully updated.');
                    \R::store($project);
                }
                else {
                    $this->app->flash('project-action', 'New project '.$project->name.' has been succesfully created.');
                    \R::exec( 'INSERT INTO projects (`id`, `name`, `link`) VALUES  (1, \''.mysql_escape_string($project->name).'\',\''.mysql_escape_string($project->link).'\')');
                }

                $this->redirect(PROJECT_FOLDER);
            }
        }


        $this->render('project/edit', array(
            'project' => $project,
            'errors'  => $errors
        ));
    }

    public function removeAction() {
        /** @var \App\Model\Project $project */
        $project = \R::load('projects', 1);
        if ($project->id) {
            $lastParser = $project->getLastParser();
            if ( $lastParser && $lastParser->getStatus() != \App\Model\Parser::STATUS_FINISH) {
                $this->app->flash('project-action',\t('PROJECT_REMOVE_PARSER_NOT_FINISHED'));
            }
            else {
                \R::trash($project);
                $this->app->flash('project-action','Project '.$project->name.' has been removed successfully');
            }
        }

        $this->redirect(PROJECT_FOLDER);
    }

    public function dashboardAction() {
        $project = $this->getProject(1);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }

        $charts = array(
            'links_on_start' => array(),
            'links' => array(),
            'ratio' => array()
        );

        foreach ($project->ownParser as $item) {
            $time = strtotime($item->started) * 1000;
            $charts['links_on_start'][] = array($time, $item->links_on_start);
            $charts['links'][] = array($time, $item->links);
            $nofollow = $item->nofollow ? $item->nofollow : 1;
            $dofollow = $item->dofollow ? $item->dofollow : 1;
            $charts['ratio'][] = array($time, (float)($dofollow / $nofollow));
        }


        $this->render('project/dashboard', array(
            'layout' => 'layouts/project.php',
            'project' => $project,
            'charts'  => $charts
        ));

    }


    public function backlinksAction()
    {
        $project = $this->getProject(1);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }

        $parser = $project->getLastParser();

        if (!$parser) {
            $parser = \R::dispense('parser');
        }

        $this->render('project/backlinks', array(
            'layout' => 'layouts/project.php',
            'project' => $project,
            'parser'  => $parser,
        ));
    }

    public function settingsAction()
    {
        $project = $this->getProject(1);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }

        if ($this->request()->isPost()) {

            $project->ose = (int)(bool)$this->request()->post('ose');
            $project->alexa = (int)(bool)$this->request()->post('alexa');
            $project->googlepr = (int)(bool)$this->request()->post('googlepr');
            $project->googlecache = (int)(bool)$this->request()->post('googlecache');
            $project->enable_cron = (int)(bool)$this->request()->post('enable_cron');

            $project->social = (int)(bool)$this->request()->post('social');

            $project->ose_key = $this->request()->post('ose_key');
            $project->ose_member_id = $this->request()->post('ose_id');
            if (filter_var($this->request()->post('mail'), FILTER_VALIDATE_EMAIL)) {
                $project->mail = $this->request()->post('mail');
            }


            if ($project->enable_cron) {
                $cron = new \jqCron\Cron($this->request()->post('cron-settings'));

                if (!in_array($cron->getType(), array($cron::TYPE_DAY, $cron::TYPE_MONTH, $cron::TYPE_WEEK))) {
                    $cron->setCron('0 0 * * 2');//set every tuesday
                }

                $project->cron = $cron->getCron();
                $project->cron_next_run = (\Cron\CronExpression::factory($project->cron)->getNextRunDate()->format('U'));
            }
            else {
                $project->cron = '';
            }


            \R::store($project);
        }

        $this->render('project/settings', array(
            'layout' => 'layouts/project.php',
            'project' => $project
        ));


    }


}