<?php

namespace App\Model;

use \SEOstats\Services\Social as Social;


/**
 * Class Backlink
 * @package App\Model
 * @author Artsiom Malkovich (skype: artmalk)
 */
class Backlink extends \RedBean_SimpleModel {

    protected $parserRedirect = 0;
    protected $xpath = null;
    protected $host = null;

    protected $parsed = array(
        'allLinks' => null
    );

    protected $newErrors = array();

    protected $projectLink = null;

    protected $errors = array();

    const EXTERNAL_LINK = 'external';
    const INTERNAL_LINK = 'internal';

    public function open()
    {
        $this->bean->link = trim($this->bean->link);//if someone use sql to add. Impossible situation at all.
        $this->host = self::getHostWithoutWWW($this->bean->link);
        $this->projectLink = unserialize($this->bean->project_link);
        $this->errors = unserialize($this->bean->errors);

    }

    public function update()
    {
        $this->bean->project_link = serialize($this->projectLink);
        $this->bean->errors = serialize($this->newErrors);
    }


    public function parse()
    {
        libxml_use_internal_errors(true);
        $dom = new \DomDocument();
        $page = $this->loadLink();

        if ($page['html']) {
            $dom->loadHTML($page['html']);
            libxml_use_internal_errors(false);
            $this->xpath = new \DomXPath( $dom );
            $this->bean->internal_links = count($this->getLinksByType(self::INTERNAL_LINK));
            $this->bean->external_links = count($this->getLinksByType(self::EXTERNAL_LINK));

            $projectLinks = $this->getProjectLinks();
            $firstLink = reset($projectLinks);


            if ($firstLink) {
                if ($this->projectLink) {
                    $firstLink['new'] = false;
                    if (!$firstLink['dofollow'] && $this->projectLink['dofollow']) {
                        $firstLink['nofollow_new'] = true;
                        $this->projects->addNotice($this, 'nofollow');
                    }
                }
                $this->projectLink = $firstLink;
            }
            else {
                if ($this->projectLink) {
                    $this->projectLink['new'] = false;
                    $this->projectLink['found'] = false;
                    $this->projects->addNotice($this, 'nofound');
                }
            }

            $this->bean->ip = gethostbyname($this->host);
            $this->bean->code = $page['code'];

            if ($this->bean->projects->googlecache) {
                $this->bean->google_cache = $this->getGoogleCacheStatus();
            }


            if ($this->bean->projects->googlepr) {
                $this->bean->pagerank = $this->getPageRank();
            }

            if ($this->bean->projects->ose) {
                $ose = $this->getOpenSiteExplorer();
                $this->bean->osepa = $ose['pa'];
                $this->bean->oseda = $ose['da'];
            }

            if ($this->bean->projects->alexa) {
                $this->bean->alexa = $this->getAlexaRank();
            }

            if ($this->bean->projects->social) {
                $socialCounters = $this->getSocialCounters();
                $this->bean->facebook = serialize($socialCounters['facebook']);
                $this->bean->twitter = $socialCounters['twitter'];
                $this->bean->googleplus = $socialCounters['googleplus'];
                $this->bean->delicious = $socialCounters['delicious'];
            }

        }
        else {
            $this->newErrors['http'] = 'Can\'t load backlink';
            $this->bean->internal_links = '';
            $this->bean->external_links = '';
            if ($this->projectLink) {
                $this->projectLink['found'] = false;
            }
            $this->bean->code = $page['code'];
        }
        $this->bean->updated = \R::$f->now();


    }

    /* Links to project from page
     *
     * return array
     */
    public function getProjectLinks()
    {
        $projectHost = self::getHostWithoutWWW($this->bean->projects->link);
        $results = array();
        foreach($this->getAllLinks() as $link) {
            if ($link['host'] == $projectHost ) {
                $anchor = trim(strip_tags($link['node']->nodeValue));
                if (!$anchor) {
                    $alt = $link['node']->getAttribute('alt');
                    if ($alt) {
                        $anchor = 'Link, anchor in alt: '.$alt;
                    }
                    else {
                        $images = $this->xpath->query('img', $link['node']);
                        foreach ($images as $image) {
                            $anchor = trim($image->getAttribute('alt'));
                            if ($anchor) {
                                break;
                            }
                        }
                    }
                }
                if (!$anchor) {
                    if (isset($image)) {
                        $anchor = 'Image, Alt = None';
                    }
                    else {
                        $anchor = 'Anchor or image not found';
                    }
                }
                $item = array(
                    'dofollow' =>  ($link['node']->getAttribute('rel') != 'nofollow'),
                    'nofollow_new' => false,
                    'anchor' => $anchor,
                    'href'   => $link['node']->getAttribute('href'),
                    'found'  => true,
                    'new'    => true
                );
//                $results[md5(self::formatLink($link['node']->getAttribute('href')))] = $item;
                $results[] = $item;
            }
        }
        return $results;
    }

    public function getLinksByType($type)
    {
        if ($type != self::EXTERNAL_LINK) {
            $type = self::INTERNAL_LINK;
        }
        $results = array();
        foreach ($this->getAllLinks() as $link) {
            $host = $link['host'];
            if (($host == $this->host || empty($host)) && $type == self::INTERNAL_LINK) {
                $results[] = $link;
            }
            elseif ($type == self::EXTERNAL_LINK && $host && $host != $this->host) {
                $results[] = $link;
            }

        }
        return $results;
    }

    private function loadLink()
    {
        return $this->urlRequest($this->bean->link);
    }


    private function urlRequest($link, $cookies = null)
    {
        //if loop
        if ($this->parserRedirect > 4) {
            $this->parserRedirect = 0;
            return array(
                'html' =>'',
                'code' => ''
            ) ;
        }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.1.2) Gecko/20090729 Firefox/3.5.2 GTB5');
        curl_setopt($ch, CURLOPT_URL, $link);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_ENCODING, '');

        if ($cookies) {
            curl_setopt($ch, CURLOPT_COOKIE, $cookies);
        }

        $result = curl_exec($ch);
        $http_status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if (curl_error($ch)) {
            $this->newErrors[] = curl_error($ch);
        }

        //folow location
        if(preg_match('#Location: (.*)#', $result, $r)) {
            $this->parserRedirect++;
            $l = trim($r[1]);
            $this->newErrors[] = 'Redirect to '.$l;
            preg_match_all('|Set-Cookie: (.*);|U', $result, $matches);
            $cookies = implode('; ', $matches[1]);
            $response= $this->urlRequest($l, $cookies);
            $result = $response['html'];
        }

        return array(
            'html' => $result,
            'code' => $http_status
        );
    }

    public static function formatLink($link)
    {
        if (substr($link, -1) === '/') {
            $link = substr_replace($link ,"",-1);
        }
        $link = trim($link);
        return $link;
    }

    public static function isValid($link)
    {
        if (!preg_match('/\b(?:(?:https?):\/\/|www\.)[-a-z0-9+&@#\/%?=~_|!:,.;]*[-a-z0-9+&@#\/%=~_|]/i', $link)) {
            return false;
        }
        return true;
    }

    public static function getHostWithoutWWW($link)
    {
        $host = parse_url($link, PHP_URL_HOST);
        if (substr($host,0,4) == 'www.') {
            $host = substr_replace($host, '', 0, 4);
        }
        return $host;

    }

    protected function getAllLinks()
    {
        if (is_null($this->parsed['allLinks'])) {
            $this->parsed['allLinks'] = array();
            $links = $this->xpath->query('.//a');
            foreach ($links as $link) {
                $href = $link->getAttribute('href');
                $host = $this->getHostWithoutWWW($href);
                $this->parsed['allLinks'][] = array(
                    'node' => $link,
                    'host' => $host,
                    'href' => $href
                );
            }
        }
        return $this->parsed['allLinks'];
    }

    protected function getGoogleCacheStatus ()
    {
        $url = 'http://webcache.googleusercontent.com/search?q=cache:'.str_replace('http://', '', $this->bean->link);
        $response = $this->urlRequest($url);
        if ($response['code'] == 404) {
            $result = 'Cache not found';
        }
        else {
            if(preg_match('/snapshot of the page as it appeared on ([a-z0-9 :,]+) GMT/i', $response['html'], $mdc)) {
                $result = date('Y-m-d', strtotime($mdc[1]));
            }
            else {
                $result = 'Unknown cache status';
            }
        }


        return $result;
    }

    protected function getPageRank()
    {
        try {
            $pagerank = \SEOstats\Services\Google::getPageRank($this->bean->link);
        }
        catch (\Exception $e) {
            $this->newErrors['pagerank'] = 'Error on pagerank.';
            $pagerank = 'None';
        }

        return $pagerank;
    }

    public function getOpenSiteExplorer()
    {
        //TODO: make own parser for ose. It's allows 10 domain in one time
        sleep(10);// moz allows 1 request per 10 seconds
        //Your Access ID and Secret Key
        $accessID = $this->bean->projects->ose_member_id;
        $secretKey = $this->bean->projects->ose_key;

        //Expires parameter must be set in near future because MOZ API does not allow parameters set far long in future and will return error
        $expire_in = time() + 500;

        $SignIn = $accessID."\n".$expire_in;

        $binarySignature = hash_hmac('sha1', $SignIn, $secretKey, true);
        $urlSafeSignature = urlencode(base64_encode($binarySignature));

        //What data we want to get, we calculate all numbers together
        $data = "103079215104";

        $curlURL = "http://lsapi.seomoz.com/linkscape/url-metrics/?Cols=".$data."&AccessID=".$accessID."&Expires=".$expire_in."&Signature=".$urlSafeSignature;

        $domains = array($this->bean->link);
        $domains = json_encode($domains);

        //User CURL for sending request
        $options = array(
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POSTFIELDS     => $domains
        );

        $ch = curl_init($curlURL);
        curl_setopt_array($ch, $options);
        $response = json_decode(curl_exec($ch));



        if (!is_array($response) || isset($response['error_message']) || curl_error($ch)) {
            if (isset($response['error_message'])) {
                $this->newErrors['ose'] = $response['error_message'];
            }
            elseif (curl_error($ch)) {
                $this->newErrors['ose'] = curl_error($ch);
            }
            else {
                $this->newErrors['ose'] = 'Undefined error while got OSE.';
            }

            $result = array(
                'da' => 'Error while got. Check api settings.',
                'pa' => 'Error while got. Check api settings.'
            );

        }
        else {
            $result = array(
                'da' => round($response[0]->pda, 2),
                'pa' => round($response[0]->upa, 2)
            );
        }

        curl_close( $ch );



        return $result;
    }

    public function getNoFollow()
    {
        if ($this->projectLink) {
            if ($this->projectLink['found'] && !$this->projectLink['dofollow']) {
                return 1;
            }
        }
        return 0;
    }

    public function getDoFollow()
    {
        if ($this->projectLink) {
            if ($this->projectLink['found'] && $this->projectLink['dofollow']) {
                return 1;
            }
        }
        return 0;
    }

    public function getFoundedLinks()
    {
        if ($this->projectLink) {
            if ($this->projectLink['found']) {
                return 1;
            }
        }
        return 0;
    }

    public static function getColor($link)
    {
        if (!$link['found']) {
            return 'red';
        }
        else {
            if ($link['new']) {
                return 'green';
            }
            if ($link['nofollow_new']) {
                return 'yellow';
            }
        }
    }

    protected  function getAlexaRank()
    {
        $rank = null;
        try {
            $seostats = new \SEOstats\SEOstats;
            // Bind the URL to the current SEOstats instance.
            if ($seostats->setUrl($this->bean->link)) {
                $rank =  \SEOstats\Services\Alexa::getGlobalRank();
            }
            else {
                $this->newErrors[] = 'Unable to set url for parse alexa';
            }
        }
        catch(\Exception $e) {
            $this->newErrors['alexa'] = 'Error got alexa rank.';
        }
        if (is_null($rank)) {
            $result = 'Unable to load alexa rank';
        }
        else {
            $result = $rank;
        }

        return $result;
    }

    public function getProjectLink()
    {
        return $this->projectLink;
    }

    public function getErrors()
    {
        return $this->errors;
    }


    public function getSocialCounters()
    {

        $results = array();

        try {
            // Create a new SEOstats instance.
            $seostats = new \SEOstats\SEOstats;

            // Bind the URL to the current SEOstats instance.
            if ($seostats->setUrl($this->bean->link)) {

                try {
                    $results['googleplus'] = \SEOstats\Services\Social::getGooglePlusShares();
                }
                catch (\Exception $e) {
                    $results['googleplus'] = 'Error while got';
                    $this->newErrors['googleplus'] = 'Error got googleplus:'. $e->getMessage();
                }

                try {
                    $results['twitter'] = Social::getTwitterShares();
                }
                catch (\Exception $e) {
                    $results['twitter'] = 'Error while got';
                    $this->newErrors['twitter'] = 'Error got twitter:'. $e->getMessage();
                }

                try {
                    $results['facebook'] = Social::getFacebookShares();
                }
                catch (\Exception $e) {
                    $results['facebook'] = 'Error while got';
                    $this->newErrors['facebook'] = 'Error got facebook:'. $e->getMessage();
                }

                try {
                    $results['delicious'] = Social::getDeliciousShares();
                }
                catch (\Exception $e) {
                    $results['delicious'] = 'Error while got';
                    $this->newErrors['delicious'] = 'Error got delicious:'. $e->getMessage();
                }

            }
        }
        catch (\Exception $e) {
            $this->newErrors['social'] = 'Error got social counters:'. $e->getMessage();
            $results = array(
                'googleplus' =>  'Error while got',
                'twitter'    =>  'Error while got',
                'facebook'   =>  'Error while got',
                'delicious'  =>  'Error while got'
            );
        }


        return $results;
    }

}